/*
 * erl_marshal.h
 */

#ifndef _ERL_MARSHAL_H
#define _ERL_MARSHAL_H

#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>

#include "httpd.h"
#include "http_config.h"
#include "http_core.h"
#include "http_log.h"
#include "http_main.h"
#include "http_protocol.h"
#include "http_request.h"
#include "util_script.h"
#include "multithread.h"

#define MAXIOVECS 1000

typedef struct {
  struct iovec iovecs[MAXIOVECS];
  unsigned int index;
  char *receiving;
  size_t length;
} erl_message;

/* Initialize an erl_message */
void init_message (erl_message *message);

/* Add raw data to a message (and include length information) */
void add_raw (erl_message *message, const void *data, size_t len,
	      request_rec *r);

#define add_string(message,str,r) add_raw ((message), (str), strlen(str), (r))

/* Add an integer to a message */
void add_integer (erl_message *message, int value, request_rec *r);

/* Read length and raw data from a message */
const void *read_raw (erl_message *message, size_t *len);

/* Return a null-terminated string from a message */
const char *read_string (erl_message *message, request_rec *r);

/* Read an integer from a message */
int read_integer (erl_message *message);

/* Send on a file descriptor. If add_length is not 0, then the length of the
 * message is first sent on the FD. Return 0 if the message was written
 * succesfully, 1 otherwise. */
int send_message (int fd, erl_message *message, int add_length,
		  request_rec *r);

/* Receive a message on a file descriptor. If receive_length is 0, then
 * the length is automatically extracted from the FD. Return 0 if
 * the message was read succesfully, 1 otherwise. */
int read_message (int fd, erl_message *message, size_t receive_length,
		  request_rec *r);

#endif
